#ifndef _FICHIER_H
#define _FICHIER_H
#define FIN_DE_FICHIER EOF
/* Topic: Documentation du module Fichier
 * Ce module permet la gestion de flux de fichier en lecture
 * en utilisant certaines oprations dfinies.
 */
 
typedef FILE Fichierl;

/**Function: fichierlOuvrir
 * Fonction permettant l'ouverture d'un fichier par rapport  son chemin d'accs
 * 
 * Paramtre:
 * c - chemin d'accs au fichier
 *
 * Retour: 
 *  Pointeur vers un fichierl
 *
 * Erreur:
 *  Si on ne trouve pas le fichier sur le disque dur, le pointeur NULL est renvoy
 */
Fichierl* fichierlOuvrir(char* c);

/**Function: fichierlLireEntier
 * Fonction permettant de lire un entier dans un flux de type *Fichierl*
 *
 * Paramtres:
 * f - fichier  lire
 * i - l'entier lu sera crit dans i
 *
 * Retour:
 *  Si l'entier a t lu, la fonction retourne EXIT_SUCCESS
 *
 * Erreur:
 *  Si la fin de fichier a t atteint avant d'obtenir un entier, FIN_DE_FICHIER est retourn
 */
int fichierlLireEntier (Fichierl* f, int* i);

/**Function: fichierlLireCaractere
 * Fonction permettant de lire un caractre dans un flux de type *Fichierl*. Les caractres spciaux
 * tel que \n ou \0 sont ignors
 * Paramtres:
 * f - fichier  lire
 * lu - le caractre lu sera crit dans i
 *
 * Retour:
 *  Si l'entier a t lu, la fonction retourne EXIT_SUCCESS
 *
 * Erreur:
 *  Si la fin de fichier a t atteint avant d'obtenir un caractre, FIN_DE_FICHIER est retourn
 */
int fichierlLireCaractere (Fichierl*f, char* lu);

/**Function: fichierlLireLigne
 * Fonction permettant de lire une ligne de caractres dans un flux de type *Fichierl*. Les caractres spciaux
 * tel que \n ou \0 sont ignors
 * Paramtres:
 * f - fichier  lire
 * lu - on crira dans cette chaine
 *
 * Retour:
 *  Si on a bien obtenu un caractre, la fonction retourne EXIT_SUCCESS
 *
 * Erreur:
 *  Si la fin de fichier a t atteint avant d'obtenir un caractre, FIN_DE_FICHIER est retourn.
 *  Si le fichier pointe sur NULL, EXIT_FAILURE est retourn
 */
int fichierlLireLigne (Fichierl* f, char* c);

/**Function: fichierlLireEgale
 * Fonction permettant de lire une ligne de caractres dans un flux de type *Fichierl* jusqu' ce que le symbole = est rencontr. Les caractres spciaux
 * tel que \n ou \0 sont ignors
 * Paramtres:
 * f - fichier  lire
 * lu - on crira dans cette chaine
 *
 * Retour:
 *  Si il n'existe plus de ligne, la fonction retourne EXIT_SUCCESS.
 *  c a t modifi tel que son dernier caractre soit \0
 *
 * Erreur:
 *  Si la fin de fichier a t atteint avant d'obtenir un caractre, FIN_DE_FICHIER est retourn.
 *  Si le fichier pointe sur NULL, EXIT_FAILURE est retourn
 */
int fichierlLireEgale (Fichierl* f, char* c);

/**Function: fichierlFermer
 * Fonction permettant la fermeture d'un *fichierl* 
 * 
 * Paramtre:
 * f - flux de fichier
 *
 * Retour: 
 *  EXIT_SUCCESS si on a bien russi  fermer le fichier
 *
 * Erreur:
 *  Dans le cas d'un problme, EXIT_FAILURE est retourn.
 */
int fichierlFermer(Fichierl *f);

#endif
