#ifndef _NIVEAU_H
#define _NIVEAU_H



#include "../liste/liste.h"
#include "../evenement/touche/touche.h"
#include "../evenement/temps/temps.h"

#include "../joueur/type_joueur/joueur.h"
#include "../diamant/type_diamant/diamant.h"
#include "../pierre/type_pierre/pierre.h"
#include "../explosion/type_explosion/explosion.h"
#include "../ennemi/type_ennemi/ennemi.h"

#include "../joueur/liste_joueur/liste_joueur.h"
#include "../diamant/liste_diamant/liste_diamant.h"
#include "../pierre/liste_pierre/liste_pierre.h"
#include "../ennemi/liste_ennemi/liste_ennemi.h"
#include "../explosion/liste_explosion/liste_explosion.h"

#include "../slim/slim.h"
#include "../mur_qui_bouge/mur_qui_bouge.h"
#include "../fichier/fichier.h"

/* Topic: Documentation du module Niveau
 * Ce module permet l'utilisation du type abstrait niveau, notamment par la possibilit de charger
 * un niveau du disque dur.
 */

typedef enum {
        ELEMENT_DIAMANT,
        ELEMENT_ENNEMI,
        ELEMENT_JOUEUR,
        ELEMENT_PIERRE,
        ELEMENT_MUR,
        ELEMENT_MUR_QUI_BOUGE,
        ELEMENT_VIDE,
        ELEMENT_TERRE,
        ELEMENT_BORD,
        ELEMENT_SLIM,
        ELEMENT_PORTE_DE_FIN,
        ELEMENT_EXPLOSION
        } Niveau_element;

typedef enum {
        NIVEAU_ETAT_INTRODUCTION,
        NIVEAU_ETAT_EN_COURS,
        NIVEAU_ETAT_GAGNE,
        NIVEAU_ETAT_MORT_DES_JOUEURS,
        NIVEAU_ETAT_PLUS_DE_TEMPS,
        NIVEAU_ETAT_GAME_OVER_GAGNE,
        NIVEAU_ETAT_GAME_OVER_PERD
        } Niveau_etat;

typedef enum {
        NIVEAU_PORTE_FERMEE,
        NIVEAU_PORTE_OUVERTE
        } Niveau_porte;
        
                
typedef struct {
        Niveau_element* matrice;
        int nombre_de_diamant_restant;
        int nombre_de_joueur;
        Liste_pierre liste_pierre;
        Liste_diamant liste_diamant;
        Liste_ennemi liste_ennemi;
        Liste_explosion liste_explosion;
        
        Liste_joueur liste_joueur;
        
        Slim          slim;
        Mur_qui_bouge mur;
        int longueurx;
        int longueury;
        int temps_restant;
        Niveau_etat etat;
        Niveau_porte porte;
        
        /*private*/
        Temps private_temps;
        int numero_niveau;
        } Niveau;
        
        
/**Function: niveauInitialiser
 * Cette fonction initialise un niveau, notamment en initialisant les listes que contient celui-ci
 *
 * Paramtre:
 * niveau - le niveau que l'on souhaite initialiser
 */
void niveauInitialiser(Niveau* niveau);

void niveauReinitialiser(Niveau* niveau);


/**Function: niveauDetruire
 * Cette fonction dtruire un niveau, notamment en dtruisant toutes les listes qu'il contient
 *
 * Paramtre:
 * niveau - le niveau  dtruire
 */
void niveauDetruire(Niveau* niveau);

void niveauDetruireSaufJoueur(Niveau* niveau);
     
/**Function: niveauLireMatrice
 * Permet de lire un lment du niveau se trouvant au position i et j de celui-ci.
 *
 * Paramtres:
 * niveau - le niveau ou l'on souhaite lire
 * i - position horizontale du niveau
 * j - position verticale du niveau
 *
 * Retour:
 * L'lment lu, qui se trouve parmi les valeurs:
 *    -  ELEMENT_DIAMANT,
 *    -  ELEMENT_ENNEMI,
 *    -  ELEMENT_JOUEUR,
 *    -  ELEMENT_PIERRE,
 *    -  ELEMENT_MUR,
 *    -  ELEMENT_MUR_QUI_BOUGE,
 *    -  ELEMENT_VIDE,
 *    -  ELEMENT_TERRE,
 *    -  ELEMENT_BORD,
 *    -  ELEMENT_SLIM,
 *    -  ELEMENT_PORTE_DE_FIN,
 *    -  ELEMENT_EXPLOSION
 *
 * Erreur:
 *  Dans le cas d'une tentative de lecture hors borne, un message d'erreur est affich sur la sortie
 *  stderr, et ELEMENT_VIDE est renvoy.
 */
Niveau_element niveauLireMatrice(Niveau* niveau, int i, int j);

/**Function: niveauEcrireMatrice
 *Permet d'crire un lment dans un niveau  une certaine position
 *
 * Paramtres:
 * niveau - le niveau ou l'on souhaite crire
 * i - position horizontale du niveau
 * j - position verticale du niveau
 * element - l'lment que l'on souhaite ajouter
 *
 * Erreur:
 *  Dans le cas d'une tentative d'criture hors borne, un message d'erreur est affich sur la sortie 
 *  stderr
 */
void niveauEcrireMatrice(Niveau* niveau, int i, int j, Niveau_element element);

/* dcremente lenombre de diamant restant  prendre*/
void niveauDecrementerDiamantRestant(Niveau* niveau);

/* rcupere le nombre de diamant restants  prendre*/
int  niveauLireNombreDiamantRestant(Niveau* niveau);

/* fait voluer le temps du niveau*/
void niveauEvoluerTemps(Niveau* niveau);

/* lit le temps restant en int (en seconde)*/
int niveauLireTempsRestant(Niveau* niveau);

/*accesseurs*/
int niveauLireLongueurX(Niveau* niveau);
int niveauLireLongueurY(Niveau* niveau);

Niveau_etat niveauLireEtat(Niveau* niveau);
void niveauEcrireEtat(Niveau* niveau, Niveau_etat etat);


int niveauLireNombreDeJoueur(Niveau* niveau);


/*accesseurs des listes des objets*/
Liste_pierre* niveauLireListePierre(Niveau* niveau);
Liste_diamant* niveauLireListeDiamant(Niveau* niveau);
Liste_ennemi* niveauLireListeEnnemi(Niveau* niveau);
Liste_joueur* niveauLireListeJoueur(Niveau* niveau);
Liste_explosion* niveauLireListeExplosion(Niveau* niveau);

Mur_qui_bouge* niveauLireMurQuiBouge(Niveau* niveau);
Slim* niveauLireSlim(Niveau* niveau);


Niveau_porte niveauLirePorte(Niveau* niveau);

void niveauEcrirePorte(Niveau* niveau, Niveau_porte porte);


/*permet de charger un niveau au format fichier dans un niveau au format structure*/
int niveauChargement(Niveau* niveau, char*); 



#endif
